// file: Data.java
//

// import necessary java libraries
//
import javax.swing.*;
import java.awt.*;
import java.awt.GraphicsEnvironment.*;

/**
 * contains data in array format for the input and output values
 * also contains flags to be checked before and after perming FFT analysis
 */
public class Data {
    
    // *********************************************************************
    //
    // declare global variables and components
    //
    // *********************************************************************
    
    // declare class constants
    //
    public final static int FFT_LENGTH = 1024;
    public final static int POWER_OF_FFT = 9;
    
    /**
     * index of the t = 0 element in the array
     */
    int zero = 0;

    /**
     * number of samples in the input data
     */
    int numsamples = 0;

    /**
     * maximum number of samples 
     */
    int maxsamples = 0;
    
    /**
     * power of FFT
     */
    int fftExponent = 0;
    
    /**
     * FFT length based on the number of samples
     */
    int capacity = 1;
    
    /**
     * sample frequency of the input data
     */
    double sampleFrequency = 2.0;
    
    /**
     * signal window duration
     */
    double duration = 4.0;

    /**
     * holds input signal values (x1 y1, x2 y2 ... )
     */
    double inputSignal[] = null;

    /**
     * holds system response values (x1 y1, x2 y2 ... )
     */
    double systemResponse[] = null;

    /**
     * holds input values for FFT
     */
    double fftData[] = null;
    
    /**
     * holds values for fft data multiplied by the window function
     */
    double windowedFFTData[] = null;
    
    /**
     * holds output values after FFT performed
     * in form a + jb
     */
    double outputData[] = null;
    
    /**
     * holds window function data
     */
    double windowFunction[] = null;
    
    /**
     * signifies if fft ready to be performed
     */
    boolean inputDataReady = false;
    
    /**
     * signifies if input data has been cleared
     */
    boolean inputCleared = false;
    
    /**
     * signifies that user wants to compute the spectrum of the signal
     */
    boolean performFFT = false;
    
    /**
     * Color for the backgroud of the plots.  This had to be 
     * determined manually to match the system background color
     */
    final static Color backgroundColor = new Color(204, 204, 204);
    
    /**
     * Set the border font up for all of the borders
     */
    Font borderFont = new Font("Serif", Font.PLAIN, 14);
    
    // *********************************************************************
    //
    // declare class constructors
    //
    // *********************************************************************

    /**
     * default constructor for the data object
     */
    Data() {
	numsamples = FFT_LENGTH;
    }

    /**
     * initializes the number of samples the data object will store
     *
     * @param len number of samples to be set
     */
    Data(int len) {
	numsamples = len;
    }
    
    // *********************************************************************
    //
    // declare class methods
    //
    // *********************************************************************

    /**
     * writes all ones into the window function so that all values of input
     * will pass through
     */
    public void initializeWindowFunction() {
       
	// make sure that the window function array is iniyialized
	//
	if (windowFunction == null) {
	    windowFunction = new double[capacity];
	}

	for ( int i = 0; i < capacity; i++ ) {
	    windowFunction[i] = 1.0;
	}
    }
    
    /**
     * returns the size of the data object
     *
     * @return the number of samples in the data object
     */
    public int getSize() {
	return numsamples;
    }
    
    /**
     * initializes the output data array
     */
    public void initializeArrays() {
	
	// allocate memory to the arrays
	//
	fftData = new double[capacity];  
	windowFunction = new double[capacity];
	inputSignal = new double[maxsamples];  
	systemResponse = new double[maxsamples];
	windowedFFTData = new double[capacity];
	outputData = new double[capacity << 1];
    }
    
    /**
     * multiplies the input data by the window function
     */
    public void applyWindow() {

	// check for valid data before windowing
	//
	if (windowedFFTData == null || fftData == null || 
	    windowFunction == null) {
	    return;
	}

	// apply the window to the data samples
	//
	for ( int i=0; i < numsamples; i++ ) {
	    windowedFFTData[i] = fftData[i] * windowFunction[i];
	}
    }
    
    /**
     * zero pad the windowed data
     */
    public void zeroPad() {

	// check for valid data before zero padding
	//
	if (windowedFFTData == null) {
	    return;
	}

	// zero out the window funztion
	//
	for ( int i = numsamples; i < capacity; i++ ) {
	    windowedFFTData[i] = 0.0;
	}
    }
    
    /**
     * shifts starting point of the fft window to time = 0
     */ 
    public void timeShift() {
	
	// declare local variables
	//
	int k = 0;

	// check for valid data before time shifting
	//
	if (windowedFFTData == null) {
	    return;
	}

	// allocate memory to the time shifting array
	//
	double timeShifted[] = new double[windowedFFTData.length];

	// provided that the transition (t = 0) cannot be found the
	// exit the method knowing that the phase will not be correct
	//
	if (zero == -1) {
	    return;
	}

	// move data from t = 0 into the shifted time array
	//
	for (int i = zero; i < windowedFFTData.length; i++, k++) {
	    timeShifted[k] = windowedFFTData[i];
	}

	// move data from t < 0 into the shifted time array
	//
	for (int i = 0; i < zero; i++, k++) {
	    timeShifted[k] = windowedFFTData[i];
	}

	// data that the fft is performed on is time shifted, hence
	// set this array equal to the time shifted array
	//
	windowedFFTData = timeShifted;
    }
}

// 
// end of file











