// file: DisplayCanvas.java
//

// import necessary java libraries
//
import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.*;
import java.lang.Math;
import java.util.*;

/**
 * DisplayCanvas draws the convolved waveform using the graph package
 *
 * hierarchy: G2Dint->OutGridCanvas->DisplayCanvas
 */
public class DisplayCanvas extends DispGridCanvas {
    
    // *********************************************************************
    //
    // declare global variables and components
    //
    // *********************************************************************

    // declare variables for initializing the curveArray
    //
    double xentry = 0.0;
    double yentry = 0.0;
    
    // declare the signal plot gui
    //
    SystemPanel systemPanel;

    // *********************************************************************
    //
    // declare class constructors
    //
    // *********************************************************************
    
    /**
     * default class constructor
     */
    DisplayCanvas()  {
	super();
    }
    
    /**
     * initializes the system and data objects to be used later in 
     * plotting the convolved waveform
     *
     * @param d output data samples from the convolved waveform
     * @param sys input system panel object
     */
    DisplayCanvas(Data d, SystemPanel sys)  {

	super(d);    
	systemPanel = sys;
    }

    // *********************************************************************
    //
    // declare class methods
    //
    // *********************************************************************


    /**
     * method clears the drawing area
     */
    public void clearDrawingArea() {

	// initializes the variables related to custom plot drawing
	//
	initCustom();

        // set the current limit to one to tell the graph package to 
        // only draw one pixel on the screen
        //
        currLimit = 1;

        // draw waveform to canvas
        //
        updateScreen();
    }

    /**
     * draws convolved waveform given the signal and system pixel points
     *
     * @param sigpix signal plot pixel points
     * @param syspix system plot pixel points
     * @param t time interval to compute the convolved waveform at
     */
    public void drawWave(int[] sigpix, int[] syspix, double t) {
	
	// declare local variables
	//
	double sumofproduct = 0.0;
	double sigamp = 0.0;
	double sysamp = 0.0;

	//  allocate memory inorder to store data points only if necessary
	//
	if(curveArray == null) {
	    curveArray = new double[2 * (datarect.width + 1)];
	}

	// set the minimum and maximum time limits for the x-axis
	//
	xaxis.maximum = xaxis.getXMax();
	xaxis.minimum = xaxis.getXMin();

	int num_its = sigpix.length >> 1;

	// convert to amplitude
	//
	double xratio = (xaxis.maximum - xaxis.minimum) / datarect.width;
	double yratio = (yaxis.maximum - yaxis.minimum) / datarect.height;
	
	// loop over all points and take the sum of the products
	//
	 for (int i = 0 ; i < num_its; i++) {
	    sigamp =  yaxis.maximum - (sigpix[2 * i + 1] * yratio);
	    sysamp =  yaxis.maximum - (syspix[2 * i + 1] * yratio);
	    sumofproduct += (sigamp * sysamp);
	}

	xentry = t;
	yentry = sumofproduct;

	// append the sum to array to draw
	//
	curveArray[currLimit * 2] = (double)xentry;
	curveArray[currLimit * 2 + 1] = (double)yentry;
	currLimit++;

	// draw the waveform to the canvas
	//     
	updateScreen();
    }
    
    /**
     * draws convolved waveform till time 't' given convolved output 
     * at the time t
     *
     * @param conv_output convolved output waveform
     * @param t time interval to compute the convolved waveform at
     */
     public void drawWave(double conv_output, double t) {
	
	if(curveArray == null) {
	    curveArray = new double[2 * (datarect.width + 1)];
	}

	// append the sum to array to draw
	//
	curveArray[currLimit * 2] = (double)t;
	curveArray[currLimit * 2 + 1] = (double)conv_output;
	currLimit++;

	// draw the waveform to the canvas
	//     
	updateScreen();
     }
}

//
// end of file



