// file: FFT.java
//

/**
 * class FFT computes the fft of a set of samples passed
 */
public class FFT {
    
    // *********************************************************************
    //
    // declare global variables and components
    //
    // *********************************************************************
   
    // Class that contains the data
    //
    Data data;
    
    public final static double ISIP_INV_SQRT2 = 0.707106781188;
    public final static double ISIP_TWOPI = 2 * Math.PI;
    
    // maximum length of data array
    //
    int FFT_LENGTH;
    int POWER_OF_FFT;
    
    /**
     * arrays to hold the fft twiddle factors
     */
    double srWr[];

    /**
     * arrays to hold the fft twiddle factors
     */
    double srWi[];
    
    
    //****************************************************************
    //
    // declare class constructors
    //
    //*****************************************************************
    
    /**
     * this method is the filter method
     *
     * @param d samples for which the fft needs to be computed
     */
    FFT (Data d) {
	
	// initialize data so that it can be accessed
	data = d;
	
	FFT_LENGTH = data.capacity;
	POWER_OF_FFT = data.fftExponent;
	
	// initialize twiddle factors for fft
	//
	srWr = new double[FFT_LENGTH];
	srWi = new double[FFT_LENGTH];
	srInit();
    }
    
    //****************************************************************
    //
    // declare class methods
    //
    //****************************************************************
    
    /**
     * creates lookup tables for sin and cosine terms. 
     *
     * @return true
     */
    public boolean srInit() {
	
	// define local variables
	//
	double  q = ISIP_TWOPI / FFT_LENGTH;
	double t;
	int  i;
	
	if (FFT_LENGTH != srWr.length) {
	    
	    // resize the twiddle arrays
	    //
	    srWr = new double[FFT_LENGTH];
	    srWi = new double[FFT_LENGTH];
	    
	    q = ISIP_TWOPI / FFT_LENGTH;
	    
	    // compute the lookup table entries
	    //
	    for (i = 0; i < FFT_LENGTH; ++i) {
		t = q*i;
		srWr[i] = Math.cos(t);
		srWi[i] = Math.sin(t);
	    }
	}
	
	// exit gracefully
	//
	return true;     
    }
    
    /**
     * this method computes exponent = log (base) of order;
     * it returns false if order of the fourier transform requested for is
     * not a power of base.
     *
     * @param exponent (output) the required exponent
     * @param base (input) base provided by the user  
     * @return boolean value indicating status
     */
    public boolean isPower(int exponent, int base) {
	
	// define local variables
	//
	int  i = 1;
	exponent = 0;
	
	while (i < FFT_LENGTH){
	    i = (int)(base*i);
	    exponent = (int)(exponent + 1);
	}
	
	// if not power of two return false
	//
	if (i != FFT_LENGTH){
	    return false;
	}
	
	// exit gracefully
	//
	return true;  
    }
    
 
    /**
     * This method computes the fourier transform using the split-radix
     * algorithm. 
     * This code is based on the code by G.A.Sitton of the Rice University.
     * The theory behind the implementation closely follows the description of 
     * the split-radix algorithm in:
     *
     * J.G.Proakis, D.G.Manolakis, "Digital Signal Processing - Principles,
     * Algorithms, and Applications" 2nd Edition, Macmillan Publishing Company,
     * New York, pp. 727-730, 1992.
     *
     * @param output (output) output data array 
     *                      length of output array will always be 2*N
     *                      memory is allocated internally, not by the calling
     *                      program.
     * @param input (input) input data array
     *                      length of input array is 2*N
     *                      memory is allocated by the calling
     *                      program.
     *
     * @return boolean indicating status of computation
     */
    public boolean srComplex(double output[], double input[]) {
	
	// declare local variables
	//
	int  i, j, k, m, q, ii, i0, i1, i2, i3, is, id, temp;
	double cc1, ss1, cc3, ss3, r1, r2, r3, s1, s2, t;
	int n1, n2, n4;
	
	// update the FFT length
	//
	FFT_LENGTH = data.capacity;
	
	// check if inout data is a power of 2
	//
	if (isPower((int)FFT_LENGTH, (int)2) == false) {
	    //       ErrorHandler error_handler = new ErrorHandler();
	    //       error_handler.error(1006);
	    System.out.println("ERROR: data for FFT is not a power of 2!");
	    return false;
	}
	
	// initialize the twiddle factors if necessary
	//
	srInit();
	
	// initialize temporary memory
	//
	double srTemp[] = new double[FFT_LENGTH];
	
	// copy input real data to temporary memory and input imaginary data
	// into output so that the input data is retained
	// after calculations.
	//
	for (k = 0; k < FFT_LENGTH; ++k) {
	    srTemp[k] = input[k*2];
	    output[k] = input[k*2+1];
	}
	
	// bit reversal routine
	//
	n1 = FFT_LENGTH - 1;
	n2 = FFT_LENGTH >> 1;
	for (i = j = 0; i < n1; ++i, j += k) {
	    if (i < j) {
		t = srTemp[j];
		srTemp[j] = srTemp[i];
		srTemp[i] = t;
		t = output[j];
		output[j] = output[i];
		output[i] = t;
	    }
	    k = n2;
	    while (k <= j) {
		j -= k;
		k = k >> 1;
	    } 
	}
	
	// Length two butterflies 
	//
	is = 0;
	id = 1;
	n1 = FFT_LENGTH - 1;
	
	do {
	    id = id << 2;      
	    for (i0 = is; i0 < FFT_LENGTH; i0 += id) {
		i1 = i0 + 1;
		
		r1 = srTemp[i0];
		srTemp[i0] = r1 + srTemp[i1];
		srTemp[i1] = r1 - srTemp[i1];
		
		r1 = output[i0];
		output[i0] = r1 + output[i1];
		output[i1] = r1 - output[i1]; 
	    }
	    temp = id - 1;
	    is = temp << 1;
	} while (is < n1);
	
	// L shaped butterflies   
	//
	n2 = 2;
	for (k = 2; k <= POWER_OF_FFT; ++k) {
	    temp = n2;
	    n4 = temp >> 1;
	    n2 = n2 << 1;
	    is = 0;
	    temp = n2;
	    id = temp << 1;
	    
	    // Unity twiddle factor loops 
	    //
	    do {
		for (i0 = is; i0 < n1; i0 += id) {
		    i1 = i0 + n4;
		    i2 = i1 + n4;
		    i3 = i2 + n4;   
		    r3 = srTemp[i2] + srTemp[i3];
		    r2 = srTemp[i2] - srTemp[i3];
		    r1 = output[i2] + output[i3];
		    s2 = output[i2] - output[i3];
		    
		    srTemp[i2] = srTemp[i0] - r3;
		    srTemp[i0] = srTemp[i0] + r3;
		    srTemp[i3] = srTemp[i1] - s2;
		    srTemp[i1] = s2 + srTemp[i1];
		    
		    output[i2] = output[i0] - r1;
		    output[i0] = output[i0] + r1;
		    output[i3] = output[i1] + r2;
		    output[i1] = output[i1] - r2;
		    
		} 
		is = (id << 1) - n2;
		id = id << 2; 
		
	    } while (is < n1);
	    
	    q = ii = (int)(FFT_LENGTH / (double)n2);
	    
	    // Non-trivial twiddle factor loops 
	    //
	    for (j = 1; j < n4; ++j, q += ii) {
		cc1 = srWr[q];
		ss1 = srWi[q];
		i3 = q * 3;
		cc3 = srWr[i3];
		ss3 = srWi[i3];
		
		is = j;
		id = n2 << 1;
		
		do {
		    for (i0 = is; i0 < n1; i0 += id) {
			i1 = i0 + n4;
			i2 = i1 + n4;
			i3 = i2 + n4;
			
			r1 = (srTemp[i2] * cc1) + (output[i2] * ss1);
			s1 = (output[i2] * cc1) - (srTemp[i2] * ss1);
			r2 = (srTemp[i3] * cc3) + (output[i3] * ss3);
			s2 = (output[i3] * cc3) - (srTemp[i3] * ss3);
			
			r3 = r1 + r2;
			r2 = r1 - r2;
			r1 = s1 + s2;
			s2 = s1 - s2;
			
			srTemp[i2] = srTemp[i0] - r3;
			srTemp[i0] = r3 + srTemp[i0];
			srTemp[i3] = srTemp[i1] - s2;
			srTemp[i1] = s2 + srTemp[i1];
			
			output[i2] = output[i0] - r1;
			output[i0] = r1 + output[i0];
			output[i3] = output[i1] + r2;
			output[i1] = output[i1] - r2;
		    }
		    is = (id << 1) - n2 + j;
		    id = id << 2;
		} while (is < n1);
            }
	}
	
	// interlace the output and store in the output array
	//
	for (k = FFT_LENGTH - 1; k >= 0; --k) {
	    output[k*2+1] = output[k];
	    output[k*2] = srTemp[k];    
	}
	
	// exit gracefully
	//
	return true;
    }
 

 
    /**
     * Routine used to compute the real split-radix fft.
     * This code is based on the code by G.A.Sitton of the Rice University.
     * The theory behind the implementation closely follows the description of 
     * the split-radix algorithm in:
     *
     * J.G.Proakis, D.G.Manolakis, "Digital Signal Processing - Principles,
     * Algorithms, and Applications" 2nd Edition, Macmillan Publishing Company,
     * New York, pp. 727-730, 1992.
     *
     * @param output (output) output data array 
     *                      length of output array will always be 2*N
     *                      memory is allocated internally, not by the calling
     *                      program.
     * @param input (input) input data array
     *                      length of input array is 2*N
     *                      memory is allocated by the calling
     *                      program.
     *
     * @return boolean indicating status of computation
     */
    public boolean srfft(double[] output, double[] input) {
	

	// check for valid data before computing the fft
	//
	if (output == null || input == null) {
	    return false;
	}
	   
	// declare local variables
	//
	double   t1, t2, t3, t4, t5, t6, t;
	double   cc1, ss1, cc3, ss3;
	int     i, j, k, l, m, length_2, inc;
	int     n1, n2, n4, n8;
	int     is, id, i1, i2, i3, i4, i5, i6, i7, i8;
	
	// update the FFT length
	//
	FFT_LENGTH = data.capacity;
	POWER_OF_FFT = data.fftExponent;
	
	// check if inout data is a power of 2
	//
	if (isPower((int)FFT_LENGTH, (int)2) == false) {
	    //       ErrorHandler error_handler = new ErrorHandler();
	    //       error_handler.error(1006);
	    System.out.println("ERROR: data for FFT is not a power of 2!");
	    return false;
	}
	
	// initialize the twiddle factors if necessary
	//
	srInit();
	
	double[] srTemp = new double[FFT_LENGTH];
	
	// copy input data to temporary memory so that the input data is retained
	// after calculations.
	//
	for (k = 0; k < FFT_LENGTH; ++k) {
	    srTemp[k] = input[k];
	    output[k] = 0;
	}
	
	// bit reversal routine
	//
	n1 = FFT_LENGTH - 1;
	n2 = FFT_LENGTH >> 1;
	for (i = j = 0; i < n1; ++i, j += k) {
	    if (i < j){
		t = srTemp[j];
		srTemp[j] = srTemp[i];
		srTemp[i] = t;
	    }
	    
	    k = n2;
	    while (k <= j) {
		j -= k;
		k = k >> 1;
	    } 
	}
	
	//  First (twiddleless) L Butterflies; completes stage A in fig. 9.23   
	//
	is = 0;
	id = 4;
	n1 = FFT_LENGTH - 1;
	
	do {
	    for (i = is; i < n1; i += id) {
		t1 = srTemp[i];
		i1 = i + 1;
		t2 = srTemp[i1];
		srTemp[i] = t1 + t2;
		srTemp[i1] = t1 - t2;
	    }
	    is = (id << 1) - 2;
	    id = id << 2;
	} while (is < n1); 
	
	//  Other (twiddled) L Butterflies  
	//
	n2 = (int)2;
	for (k = 2; k <= POWER_OF_FFT; ++k) {
	    n4 = n2 >> 1;
	    n8 = n4 >> 1;
	    n2 = n2 << 1;
	    is = 0;
	    id = n2 << 1; 
	    do {
		for (i = is; i < FFT_LENGTH; i += id) {
		    i2 = i + n4;
		    i3 = i2 + n4;
		    i4 = i3 + n4;
		    t3 = srTemp[i3];
		    t4 = srTemp[i4];
		    t1 = srTemp[i];
		    t2 = t4 + t3;
		    srTemp[i4] = t4 - t3;
		    srTemp[i3] = t1 - t2;
		    srTemp[i] = t1 + t2;
		    
		    if (n4 != 1) {
			i1 = i + n8;
			i2 = n8 + i2;
			i3 = n8 + i3;
			i4 = n8 + i4;
			t3 = srTemp[i3];
			t4 = srTemp[i4];
			t1 = (t3 + t4) * ISIP_INV_SQRT2;
			t2 = (t3 - t4) * ISIP_INV_SQRT2;
			t3 = srTemp[i1];
			t4 = srTemp[i2];
			srTemp[i4] = t4 - t1;
			srTemp[i3] = -(t4 + t1);
			srTemp[i2] = t3 - t2;
			srTemp[i1] = t3 + t2;
		    }
		}
		is = (id << 1) - n2;
		id = id << 2;
	    } while (is < n1);
	    
	    inc = FFT_LENGTH/n2;
	    l = inc;
	    for (j = 1; j < n8; ++j) {
		cc1 = srWr[l];
		ss1 = srWi[l];
		
		// i1 gives the twiddle factors which are multiples of 3 and are needed
		// at the B stage in fig. 9.23
		i1 = l * 3;
		cc3 = srWr[i1];
		ss3 = srWi[i1];
		l = (j + 1) * inc;
		is = 0;
		id = n2 << 1;
		do {
		    for (i = is; i < FFT_LENGTH; i += id) {
			i1 = i + j;
			i2 = i1 +  n4;
			i3 = i2 +  n4;
			i4 = i3 +  n4;
			i5 = i - j + n4;
			i6 = i5 +  n4;
			i7 = i6 +  n4;
			i8 = i7 +  n4;
			t3 = srTemp[i3];
			t4 = srTemp[i7];
			t1 = (t3*cc1) + (t4*ss1);
			t2 = (t4*cc1) - (t3*ss1);
			t5 = srTemp[i4];
			t6 = srTemp[i8];
			t3 = (t5*cc3) + (t6*ss3);
			t4 = (t6*cc3) - (t5*ss3);
			t5 = t1 + t3;
			t3 = t1 - t3;
			t6 = t2 + t4;
			t4 = t2 - t4;
			t1 = srTemp[i6];
			srTemp[i8] = t1 + t6;
			srTemp[i3] = t6 - t1;
			t2 = srTemp[i2];
			srTemp[i4] = t2 - t3;
			srTemp[i7] = -(t2 + t3);
			t1 = srTemp[i1];
			srTemp[i1] = t1 + t5;
			srTemp[i6] = t1 - t5;
			t5 = srTemp[i5];
			srTemp[i2] = t5 + t4;
			srTemp[i5] = t5 - t4;
		    }
		    is = (id << 1) - n2;
		    id = id << 2;
		} while (is < n1);
	    }
	}
	
	// interlace the output and store in the output array
	//
	length_2 = FFT_LENGTH/2;
	for (k = 1; k <= length_2; ++k) {
	    output[k*2] = output[2*FFT_LENGTH - k*2] = srTemp[k];
	    output[2*FFT_LENGTH - k*2 + 1 ] = -srTemp[FFT_LENGTH - k];
	    output[k*2+1] = srTemp[FFT_LENGTH - k];
	}
	
	// manually set values to special points in the output data
	// zero imaginary part of dc, and N/2 point
	//
	output[0] = srTemp[0];
	output[1] = (double)0.0;
	output[FFT_LENGTH+1] = (double)0.0;
	
	for ( i=0; i<FFT_LENGTH; i++) {
	    
	    double sumr=0;
	    double sumi=0;
	    
	    for ( j=0; j<FFT_LENGTH; j++) {
		
		sumr += input[j] * Math.cos(2*Math.PI*(double)j*(double)i/(double)FFT_LENGTH);
		sumi += input[j] * Math.sin(2*Math.PI*(double)j*(double)i/(double)FFT_LENGTH);
		
	    }
	    
	    output[2*i] = sumr;
	    output[2*i+1] = sumi;
	}
	
	// exit gracefully
	//
	return true;
    }
}

//
// end of file


