// file: MagResponse.java
//

// import necessary java libraries
//
import java.awt.*;
import java.lang.*;
import javax.swing.*;
import java.awt.GraphicsEnvironment.*;

// include the graph package
//
import graph.*;

/**
 * MagResponse computes the magnitude plot of the samples passed 
 * to it by a data object and plots it using the graph package
 *
 * hierarchy: G2Dint->MagResponse
 */
public class MagResponse extends G2Dint implements Runnable {

    // *********************************************************************
    //
    // declare global variables and components
    //
    // *********************************************************************

    /**
     * constant to represent the initial frequency
     */
    public final static double INIT_FREQUENCY = (double)2.0;

    /**
     * constant to define the sleep time of the thread
     * measured in milliseconds
     */
    public final static long THREAD_SLEEP_TIME = 200;

    // declare variables used to initialize plot labels
    //
    Double MAG_X_TIC = new Double((double)0.25);
    Double MAG_Y_TIC = new Double((double)10.0);
    Double MAG_MAX = new Double((double)40.0);
    Double MAG_MIN = new Double((double)-40.0);

    public int FFT_LENGTH = 0;
  
    /**
     * log base e of 10
     */
    public final static double LOGE10 = Math.log(10);
  
    /**
     * set up a thread to take care of updating the display
     */
    Thread phaseRespThread;

    /**
     * set up a flag to determine if the display needs to be
     * updated and initialize it to false
     */
    boolean updateFlag = false;

    // data sets for the x and y-axes
    //
    DataSet dataSet, xData, yData;

    /**
     * declare the array to hold the data
     */
    double data[];

    // information about the data points
    //
    int length = 0;
    Data fftData;
    double sampleFrequency = INIT_FREQUENCY;

    // declare the axes
    //
    Axis xAxis, yAxis;

    //*****************************************************************
    //
    // declare class constructors
    //
    //*****************************************************************

    /**
     * computes the magnitude plot given the samples using the graph package
     *
     * @param fftInput input sample for computing the magnitude plot
     */
    MagResponse(Data fftInput) {

	super();

	// initialize parameters
	//
        fftData = fftInput;
        data = fftData.outputData;
        sampleFrequency = fftData.sampleFrequency;
        FFT_LENGTH = fftData.capacity;
        length = FFT_LENGTH;

	// check the maximum frequency
	//
	double delta = (double)0.1;

	if(sampleFrequency > -delta && sampleFrequency < delta) {
	    sampleFrequency = INIT_FREQUENCY;
	}

        // remove all old datasets
        //
        this.detachDataSets();

        // initialize the parameters of this class
        //
        this.drawzero = true;
        this.drawgrid = true;
        this.borderTop = 5;
        this.borderLeft = 0;
        this.borderBottom = 0;
        this.borderRight = 25;
        this.framecolor = Color.black;
        this.gridcolor = Color.lightGray;
        this.setDataBackground(Color.white);

        // attach the data to the x-axis
        //
        xAxis = this.createXAxis();
        xAxis.setTitleText(" Frequency (Hz) ");
        xAxis.setTitleFont(new Font("TimesRoman", Font.PLAIN,8));
        xAxis.setLabelFont(new Font("Helvetica", Font.PLAIN, 8));

	// set the default x-axis ranges and ticks
	//
        xAxis.setXMin((double)-sampleFrequency/(double)2.0);
        xAxis.setXMax((double)sampleFrequency/(double)2.0);
        xAxis.setXTic(MAG_X_TIC.doubleValue());

        // create the y-axis
        //
        yAxis = this.createYAxis();
        yAxis.setTitleText(" H(f) dB ");
        yAxis.setTitleFont(new Font("TimesRoman", Font.PLAIN, 8));
        yAxis.setLabelFont(new Font("Helvetica", Font.PLAIN, 8));

	// set the default y-axis ranges and ticks
	//
        yAxis.setYMin(MAG_MIN.doubleValue());
        yAxis.setYMax(MAG_MAX.doubleValue());
        yAxis.setYTic(MAG_Y_TIC.doubleValue());

	// reset the plot
	//
	resetPlot();

        // initialize the controlling thread and start it
        //
        phaseRespThread = new Thread(this);
        phaseRespThread.start();
    }

    // *********************************************************************
    //
    // declare class methods
    //
    // *********************************************************************

    /**
     * clear the plot for the currrent graph object
     */
    public void resetPlot() {

        // attach the dataSet to the graph
        //
	this.detachDataSets();
	double dummyArray[] = new double[10];
        dataSet = this.loadDataSet(dummyArray, 5);

        // attach the data sets the both axis
        //
        xAxis.attachDataSet(dataSet);
	xAxis.resetXRange();
        yAxis.attachDataSet(dataSet);
	yAxis.resetYRange();
    }

    /**
     *
     * this method draws the magnitude response of the signal on
     * the screen
     *
     * @return true
     */
    public boolean drawResponse() {

	// update the FFT length
	//
        FFT_LENGTH = fftData.capacity;
        length = FFT_LENGTH;
        sampleFrequency = fftData.numsamples / fftData.duration;

	// update the data pointer
	//
	data = fftData.outputData;

	// make sure that there is valid data to plot
	//
	if (fftData.outputData == null) {
	    return true;
	}
	if ((length <= 0) || (length > data.length / 2)) {
	    length = 1;
	}

	// declare local variables
        //
        double tempValue = 0.0;                          // response value
        double freqIncrement = sampleFrequency/length;   // Hz/pixel
        double freqLoc = 0.0;                            // current frequency
        double mag;

        // setup a data array to hold the output values and the pixel
        // representation of these
        //
        double dataArray[] = new double[length << 1];

        // holds index for left and right side of the magnitude spectrum
        //
        int rightSide;
        int leftSide;

        // for each magnitude sample's value calculate
        //
        int loopLength = length;

	for (int i = 0; i < loopLength; i+=2) {
	    
	    rightSide  = length + i;
	    leftSide = length - i;

	    dataArray[rightSide] = freqLoc;

	    if ( i != 0 ) {
		dataArray[leftSide] = -freqLoc;
	    } 
            freqLoc += freqIncrement;

	    // compute magnitude: sqrt(R*R  + I*I)
	    //
	    mag = Math.sqrt( (data[i+1] * data[i+1]) + (data[i] * data[i]) );

	    // normalize the data
	    //
	    mag /= length;	   

	    // take 20 * log base 10 of the magnitude
	    //
	    mag = 20 * (Math.log(mag) /  LOGE10);

	    // assign the value to the data array
	    //
	    dataArray[rightSide+1] = mag;
	    dataArray[leftSide+1] = mag;
	} 
	
        dataArray[0] = dataArray[2] - freqIncrement;
        dataArray[1] = dataArray[3];

        // attach the yData to the graph
        //
        this.yData = this.loadDataSet(dataArray, length);

        // attach the data sets the both axis
        //
        xAxis.attachDataSet(yData);
	xAxis.resetXRange();
        yAxis.attachDataSet(yData);
	yAxis.resetYRange();
    
        // setup properties of the yData
        //
        yData.linestyle = 1;
        yData.linecolor = Color.blue;
    
        // exit gracefully
        //
        return true;
    }
  
    /** 
     * this method sets the update flag causing a repaint of the
     * canvas
     *
     * @return boolean
     */
    public boolean updateScreen() {

	// set the update flag
        //
        updateFlag = true;

        // exit gracefully
        //
        return true;
    
    }

    /** 
     * this method is the body of the thread.  it will update the
     * display area when necessary.  the necessity is based on the
     * condition of the update flag
     */
    public void run() {

	// loop until the thread is terminated
        //
        while (phaseRespThread != null) {

	    // sleep for an appropriate number of milliseconds
            //
	    try {
		phaseRespThread.sleep(THREAD_SLEEP_TIME);
	    }
            catch (InterruptedException e) { }

            // update the screen if necessary
            //
            if (updateFlag == true) {

		// exit if there is no valid data to be plotted
		//
		if (fftData.outputData == null) {
		    return;
		}

		// draw the magnitude response
		//
		sampleFrequency = fftData.sampleFrequency;
		this.detachDataSets();
		this.drawResponse();
		repaint();
	
		// reset the update flag
		updateFlag = false;
	    }
	}
    }

    /**
     * this method starts or restarts the threaded object
     */
    public void start() {
    
	// see if a new thread needs to be created
        //
        if (phaseRespThread == null) {
	    phaseRespThread = new Thread(this);
	}
    
	// see if thread isAlive to determine if it is running
	//
        if (phaseRespThread.isAlive() == false) {

	    // start the thread
            //
            phaseRespThread.start();
	}

    }

    /**
     * this method stops the threaded object
     */
    public void stop() {
    
	// set the applet thread to null
        //
        phaseRespThread = null;
    }
}

// 
// end of file





