// file: OutputPanel.java
//

// import necessary java libraries
//
import javax.swing.*;
import javax.swing.border.*;
import java.awt.Color;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

// import the graph package
//
import graph.*;

// class: OutputPanel
//
// creates area for phase and magnitude response, and window function
// panels to be added to the applet
// controls the execution of the FFT
//
// hierarchy: JPanel->SubPanel->OutputPanel
//
class OutputPanel extends SubPanel implements Runnable {

    // *********************************************************************
    //
    // declare global variables and components
    //
    // *********************************************************************

    public boolean prevclear = false;
    public boolean reconvolve = false;

    Data signal, system, output;
    
    boolean animateFlag = true;
    boolean pauseFlag = false;
    boolean convolveFlag = false;
    
    int animateSpeed = 190;
    
    // declare title strings
    //
    String convolution = new String("Convolution");
    String animation = new String("Animation");

    // define gui components
    //
    DisplayCanvas outDisp;
    MovingDisplayCanvas drawingArea;
    SignalPanel signalPanel;
    SystemPanel systemPanel;

    // constant to define the sleep time of the thread
    // measured in milliseconds
    //
    public final static long THREAD_SLEEP_TIME = 200;
    
    // set up a thread to take care of updating the display
    //
    Thread outputThread;
    
    // declare the default fonts
    //
    Font currentFont = getFont();
    Font newFont = new Font(currentFont.getName(), currentFont.getStyle(), 12);

    // *********************************************************************
    //
    // declare class constructors
    //
    // *********************************************************************
   
    // method: ControlPanel
    //
    // arguments:
    //    Data signal: data samples from the input signal
    //    Data system: data samples from the system response
    //    Data output: data samples from the output panel
    //    SignalPanel sig: signal panel gui object
    //    SystemPanel imp: system panel gui object
    //
    // returns  : none
    //
    // constructor that sets the size, layout manager and adds components
    //
    OutputPanel (Data sig, Data sys, Data out, 
		 SignalPanel sigpanel, SystemPanel syspanel)  {

	super();
	
	// initialize the components of this class with the data passed in
	//
	signal = sig;
	system = sys;
	output = out;

	// initialize the signal and input panels
	//
	signalPanel = sigpanel;
	systemPanel = syspanel;

	// define the plotting objects for the animation and convolution
	//
	drawingArea = new MovingDisplayCanvas(output, signalPanel, 
					       systemPanel);
	outDisp = new DisplayCanvas(output, systemPanel);
	
	// initialize the controlling thread and start it
	//
	outputThread = new Thread(this);
	outputThread.start();
    }
    
    // *********************************************************************
    //
    // declare class methods
    //
    // *********************************************************************

    // method: setAnimate
    //
    // arguments:
    //    boolean value: flag indicating when to initiate convolution
    //
    // return   : none
    //
    // sets the animate flag to true to begin animation of the demo
    //
    public void setAnimate(boolean value) {
	
	animateFlag = value;
    }

    // method: setPause
    //
    // arguments: 
    //    boolean value: flag indicating when to pause the animation
    //
    // return   : none
    //
    // sets the animate flag to true to freeze the animation
    //
    public void setPause(boolean value) {

	pauseFlag = value;
    }
    
    // method: setSpeed
    //
    // arguments: 
    //    int value: ratio
    //
    // return   : none
    //
    // sets the animation speed for the demo
    //
    public void setSpeed(double ratio) {
	
	animateSpeed = (int)(THREAD_SLEEP_TIME * ratio - 1.0);
    }
    
    // method: convolve
    //
    // arguments: none
    // return   : none
    //
    // clears the screen and sets the convolution flag
    //
    public void convolve() {

	// wait for the current convolution process to finish before
	// starting a new process
	//
	if(convolveFlag == true) {
	    return;
	}

	// set the reconvolve flag to true to indicate that another 
	// process is about to start and reset the convolution flag
	//
	convolveFlag = false;
	reconvolve = true;

	// detach all previous data sets and zero out the indeces
	//
	outDisp.clearDrawingArea();
	drawingArea.clearDrawingArea();
	
	try {
	    outputThread.sleep(THREAD_SLEEP_TIME);
	} catch (InterruptedException e) { }

	prevclear = false;
	reconvolve = false;
	convolveFlag = true;
    }
    
    // method: addComponents
    //
    // arguments: none
    // return   : none
    //
    // adds a phase canvas, magnitude canvas, and a place holder for 
    // the controls and window function to the output panel
    // 
    public void addComponents () {
	
	constrain(this, outDisp, 0, 0, 
		  GridBagConstraints.REMAINDER, 1,
		  GridBagConstraints.BOTH,
		  GridBagConstraints.NORTHWEST, 1, 1, 5, 5, 5, 5); 
	constrain(this, drawingArea, 0, 1, 
		  GridBagConstraints.REMAINDER, 1, 
		  GridBagConstraints.BOTH,
		  GridBagConstraints.NORTHWEST, 1, 1, 5, 5, 5, 5);
    }
    
    // method: run
    //
    // arguments: none
    // return   : none
    //
    // this method is the body of the thread.  it will update the
    // display area when necessary.
    //
    public void run() {
	
	// loop until the thread is terminated
	//
	while (outputThread != null) {
	    
	    // sleep for an appropriate number of milliseconds
	    //
	    try {
		outputThread.sleep(THREAD_SLEEP_TIME - animateSpeed);
	    } catch (InterruptedException e) { }
	    
	    // compute the convolution only of the convolve flag is set
	    //
	    if (convolveFlag) {
		// rest the index and data sets of the convolved plot
		//
		outDisp.currLimit = 0;
		outDisp.detachDataSets();
		outDisp.xaxis.maximum = outDisp.xaxis.getXMax();
		outDisp.xaxis.minimum = outDisp.xaxis.getXMin();
		
		// declare arrays to store interpolated values for both signal
		//and system
		//
		double sigIpn[] = new double[2 * 
					     (drawingArea.datarect.width + 1)];
		double sysIpn[] = new double[2 * 
					     (drawingArea.datarect.width + 1)];
		double convOutput [] = new double[2 * 
					     (drawingArea.datarect.width + 1)];


		// allocate memory to hold the convolved samples which
		// are needed to plot the magnitude spetrum
		//
		output.fftExponent = 0;
		output.capacity = 1;
		output.numsamples = drawingArea.datarect.width + 1;

		while(output.capacity < drawingArea.datarect.width) {
		        output.capacity = output.capacity << 1;
			output.fftExponent += 1;
		}
		output.initializeArrays();

		// to interpolate the signal wave
		// check if the input signal is defined
		//
		if(signal.fftData != null) {

		    interpolate(signal.inputSignal[0],
			    signal.inputSignal[2 * (signal.numsamples-1)],
			    outDisp.xaxis.minimum,outDisp.xaxis.maximum,
			    signal.numsamples,signal.fftData.length,
			    signal.fftData,sigIpn);
		}

		// to interpolate the system wave
		// check if impulse signal is defined
		//
		if(system.fftData != null) {

		    interpolate(system.systemResponse[0],
			    system.systemResponse[2 * (system.numsamples-1)],
			    outDisp.xaxis.minimum,outDisp.xaxis.maximum,
			    system.numsamples,system.fftData.length,
			    system.fftData,sysIpn);
		}

		// compute data needed to plot the magnitude spectrum
		//
		int index = 0;
		double min = drawingArea.xaxis.getXMin();
		double max = drawingArea.xaxis.getXMax();
		double step = (max - min)/drawingArea.datarect.width;

		for (double time = min; time < max; time += step, index++) {
		    
		    // compute the spectrum of the convolved waveform
		    //
		    if((signal.fftData != null) && (system.fftData != null)) {
			spectrum(sigIpn, sysIpn, index, time);
		    }
		}

		// reverse the impulse signal time array
		// the sample values are in descending order of time
		// after reversing the array
		//
		// check if the impulse signal is defined
		//
		if(system.fftData != null) {
		    for(int l=0; l < drawingArea.datarect.width/2 ;l++) {
			double tempValue;
			tempValue = sysIpn[l];
			sysIpn[l] = sysIpn[drawingArea.datarect.width  - l];
			sysIpn[drawingArea.datarect.width  - l] = tempValue;
		    }
		}

		// do not draw the signal if the reconvolve flag is set
		//
		if (!reconvolve && animateFlag) {

		    // plot the input signal waveform
		    //
		    double sigmin = signalPanel.wavePanel.xaxis.getXMin();
		    double sigmax = signalPanel.wavePanel.xaxis.getXMax();
		    
		    // check if the input signal is defined
		    //
		    if((signal.fftData != null)) {

			drawingArea.drawWave(signal, sigmin, sigmax);
		    }
		}
		
		int arrayIndex = 0;
		for (double t = min; t < max; t += step , arrayIndex++) {

		    if ((animateFlag) && (convolveFlag)) {

			while (pauseFlag) {
			    try {
				outputThread.sleep(THREAD_SLEEP_TIME);
			    } catch (InterruptedException e) { }
			}
			try {
			    outputThread.sleep(THREAD_SLEEP_TIME - 
						animateSpeed);
			} catch (InterruptedException e) { }		       

			// if the user initiates convolution in the middle
			// of a computation exit gracefully from the loop
			//
			if (reconvolve) {
			    break;
			}

			// detach the previously computed data sets
			//
			if (prevclear && !reconvolve) {
			    drawingArea.detachDataSet(drawingArea.dataSet);
			}

			// draw the system response waveform
			//
			double sysmin = systemPanel.wavePanel.xaxis.getXMin();
			double sysmax = systemPanel.wavePanel.xaxis.getXMax();

			if (!reconvolve) {
			    // check if the impulse signal is defined
			    //
			    if(system.fftData != null) {

				drawingArea.shiftWave(system, t, sysmin, sysmax);
			    }
			}
			
			// draw the convolved wave form upto the current time
			//
			// check if input signal and impulse signal arrays
			// are defined
			//
			if((sigIpn != null) && (sysIpn != null)) {

			    convolution(sigIpn,sysIpn,convOutput,arrayIndex,t);

			    if (!reconvolve) {
				if(Math.abs(max) > Math.abs(min)) {
				    outDisp.drawWave(convOutput[arrayIndex],
						     (t + Math.abs(Math.abs(max) -
								   Math.abs(min))));
				}
				else {
				    outDisp.drawWave(convOutput[arrayIndex],
						     (t - Math.abs(Math.abs(max) -
								   Math.abs(min))));
				}
			    }
			}
			

			// set the clear flag to idicate that the initial 
			// plot has been drawn and can be cleared
			//
			prevclear = true;
		    }
		}
		
		

		// reset the convolve flag
		//
		convolveFlag = false;
		
		// if the animation flag was not set just display the result
		//
		arrayIndex = 0;

		if (!animateFlag) {
		    // check if input signal and impulse signal arrays
		    // are defined
		    //
		    if((sigIpn != null) && (sysIpn != null)) {

			for (double t = min; t < max; t += step ,arrayIndex++) {

			    // display the convolved waveform all at once
			    //
			    convolution(sigIpn, sysIpn, convOutput, arrayIndex,t);
			    if(Math.abs(max) > Math.abs(min)) {
				outDisp.drawWave(convOutput[arrayIndex],
						 (t + Math.abs(Math.abs(max) -
							       Math.abs(min))));
			    }
			    else {
				outDisp.drawWave(convOutput[arrayIndex],
						 (t - Math.abs(Math.abs(max) -
							       Math.abs(min))));
			    }
			}
		    }
		}
	    }
	}
    }

    // method: spectrum
    //
    // arguments:
    //   double sigArray[]: array having interpolated signal values
    //   double sysArray[]: array having interpolated system values
    //   int index: holds the index of the convolved output
    //   double time: time at which the output needs to be calculated
    //
    // return   : none
    //
    // method computes the samples needed to plot the magnitude spectrum
    // of the convolved waveform and stores them the output data object
    //
    public void spectrum(double sigArray[],double sysArray[],
			 int index, double time) {

	// declare local variables
	//
	int j = 0;
	double k = 0.0;
	double step = 0.0;
	double sumofproduct = 0.0;

	// compute the time per pixel ratio
	//
	step = (double)(outDisp.xaxis.getXMax() - outDisp.xaxis.getXMin())
	                                        / drawingArea.datarect.width;

	k = setDecimal((-(time / step)), 3);
	k = k + 1;
	j = (int) Math.ceil(k);
	
	int l = 0;

	// compute the convolution of the signal and system response
	//
	for(int i = j; i <= drawingArea.datarect.width; i++, l++) {
	    if(i >= 0) {
		sumofproduct += sigArray[l] * sysArray[i] * step;  
	    }
	}

	// store the computed convolved point in the data object
	//
	output.fftData[index] = sumofproduct ;
    }

    // method: interpolate
    //
    // arguments: 
    //   double inputMin: minimum value of x-axis for input signal
    //   double inputMax: maximum value of x-axis for input signal
    //   double outputMin: minimum value of x-axis for output signal
    //   double outputMax: maximum value of x-axis for output signal
    //   int numSamples: number of samples in the input signal
    //   int length: array length for input signal
    //   double inputArray: array holding the samples for input signal
    //   double interpolatedArray: array holding the values for interpolated
    //                             output signal
    // return   : none
    //
    // this method does the interpolation of a function given the set
    // of sample points.
    //
    public void interpolate(double inputMin, double inputMax,
			    double outputMin, double outputMax,
			    int numSamples, int length, double inputArray[], 
			    double interpolatedArray[]) {

	// declare local variables
	//
	int width = 0;
	int offset = 0;
	double delta = 0.0001;
	double sampFreq = (double)(numSamples+1)/(inputMax - inputMin);

	// time instants to calculate the interpolated output
	//
	double step = (double)(outputMax - outputMin)
	                       / drawingArea.datarect.width;

	int j = 0;
	for(double i = outputMin; i <= outputMax; i += step ,j++) {

	    // if time instant is less than minimum input time or
	    // if time instant is greater than maximum input time
	    // value at those time instants is zero
	    //
	    if((i < inputMin) || ( i > inputMax)) {
		interpolatedArray[j] = (double)0.0;
	    }

	    else {
		int k = 0;
		double sum = 0.0;
		double l = setDecimal((i * sampFreq), 3);

		for(int n = (int)(inputMin * sampFreq); 
		    n < (int)(inputMax * sampFreq); n++, k++) {

		    // if time instant equals 'k' times sampling period
		    // value at that time instant is equal to
		    // value of sample at that 'k'th sampling instance
		    //
		    if((Math.abs(l - n)) < delta) {
			sum = inputArray[k];
			n += length;
		    }

		    // convolve the input samples with sinc function 
		    // to get the values
		    //
		    else {
			double y = (Math.sin(Math.PI * sampFreq * 
				     (i - n / sampFreq)) /
				     ((Math.PI * sampFreq) * 
				     (i - n / sampFreq)));
			    sum = sum + inputArray[k] * y ;
		    }
		}

		interpolatedArray[j] = (double)sum;
	    }
	}
    }

    // method: convolution
    //
    // arguments:
    //   double sigArray[]: array having interpolated signal values
    //   double sysArray[]: array having interpolated system values
    //   double output[]: array that holds convolved values
    //   int arrayIndex: holds the index of the convolved output
    //   double t: time at which the output needs to be calculated
    //
    // return   : none
    //
    // method calculates the output by convolving the input wave with
    // the system wave
    //
    public void convolution(double sigArray[],double sysArray[],
			    double output[], int arrayIndex, double t) {
	
	// declare local variables
	//
	double sumofproduct = 0.0;
	int j=0;
	double k;
	double step =(double)(outDisp.xaxis.getXMax() -outDisp.xaxis.getXMin())
	                       / drawingArea.datarect.width;
	
	// calculate the offset for the impulse response signal
	// and get the sample values from that time instant 
	//
	k = setDecimal((-(t / step)),3);
	k = k + 1;
	
	// convert it to an integer after ceiling
	//
	j = (int) Math.ceil(k);
	
	int l=0;
	for(int i = j ; i <= drawingArea.datarect.width ; i++,l++) {
	    if(i >= 0) {
		// perfrom convolution only if the array subscript
		// is greater than zero
		//
		sumofproduct += sigArray[l] * sysArray[i] * step;  
	    }
	}
	// convolution sum is stored on output
	//
	output[arrayIndex] = sumofproduct ;
    }

    // method: setDecimal
    //
    // arguments:
    //   double doubleNumber: input digit to be rounded off
    //   int decimalPlaces: number of significant to round the input digit 
    //
    // return :
    //    double result: the rounded off input digit
    //
    // method takes in a input digit (double) and rounds the input digit
    // to the number of speified significant places specified
    //
    public double setDecimal(double doubleNumber, int decimalPlaces) {

	// declare local variables
	//
	int combine = 0;
	int wholeNumber = 0;
	int roundedDecimal = 0;

	double pastDecimal = 0.0;
	double result = 0.0;
	
	wholeNumber = (int)doubleNumber;
	pastDecimal = ((doubleNumber - wholeNumber) * 
		       Math.pow(10,decimalPlaces));
	roundedDecimal = (int) Math.round(pastDecimal);
	combine = (int)(wholeNumber * 
			Math.pow(10,decimalPlaces)+roundedDecimal);
	result = combine / Math.pow(10,decimalPlaces);

	// return the rounded off input digit
	//
	return result;
    }
}
	   
//
// end of file



















