// file: SignalDrawingCanvas.java
//

// import necessary java libraries
//
import javax.swing.*;
import java.awt.Graphics;
import java.lang.Math;
import java.util.Vector;
import java.awt.*;

/**
 * SignalDrawingCanvas allows for interactive drawing on a canvas and
 * also specifies specific signal signal that can be plotted
 *
 * hierarchy: G2Dint->SigGridCanvas->SignalDrawingCanvas
 */
public class SignalDrawingCanvas extends SigGridCanvas {

    // *********************************************************************
    //
    // declare global variables and components
    //
    // *********************************************************************

    // declare global variables
    //
    double xratio = 0.0;            // time per pixel value of the x-axis
    double yratio = 0.0;            // time per pixel value of the y-axis

    int xStart = 0;                 // starting pixel to plot from
    int xEnd = 0;                   // ending pixel to finish plotting

    double xentry = 0.0;            // x-axis entry to the curve array
    double yentry = 0.0;            // y-axis entry to the curve array

    Data data;

    // *********************************************************************
    //
    // declare class constructors
    //
    // *********************************************************************
   
    /**
     * constructor initializes the samples to be plotted in the signal panel
     *
     * @param d data samples form the signal panel
     */
    SignalDrawingCanvas(Data d)  {
	
	super(d);
	data = d;
    }
    
    // *********************************************************************
    //
    // declare class methods
    //
    // *********************************************************************
    
    /**
     * this method initialize the array before points can be stores in the
     * curve array and plotted on the graph package
     *
     * @param preset identifier of the last waveform drawn 
     */
    public void initArray(int preset) {

	// check if a custom plot is to be drawn
	//
	if(preset == 0) {
	    this.custom = true;
	} else {
	    this.custom = false;
	}	    

	// reset the number of samples in the data object
	//
	data.numsamples = 0;

	// keep track of the fact that is waveform was just drawn
	//
	presetWave = preset;

	//  set the minimum and maximum time limits for the x-axis
	//
	xaxis.maximum = xaxis.getXMax();
	xaxis.minimum = xaxis.getXMin();

	//  get the time per pixel ratio
	//
	xratio = (xaxis.maximum - xaxis.minimum) / datarect.width;
	yratio = (yaxis.maximum - yaxis.minimum) / datarect.height;

	// get the limits of the canvas
	//
	xStart = datarect.x;
	xEnd = datarect.x + datarect.width;

	//  place the location of each pixel of the line in an array
	//
	xentry = 0.0;
	yentry = 0.0;
	
	//  allocate memory inorder to store data points only if necessary
	//
	if(curveArray == null) {
	    curveArray = new double[(datarect.width + datarect.x) * 2];
	}

	// reset the current limit
	//
	currLimit = 0;

	// zero out the curve array
	//
	for(int i = 0; i < curveArray.length; i++) {
	    curveArray[i] = 0.0;
	}
    }

    /**
     * method clears the drawing area
     */
    public void clearDrawingArea() {

	// initializes the variables related to custom plot drawing
	//
	initCustom();

        // set the current limit to one to tell the graph package to 
        // only draw one pixel on the screen
        //
        currLimit = 1;

        // draw waveform to canvas
        //
        updateScreen();
    }

    /**
     * set up the graph package to draw a custom plot
     */
    public void drawCustom() {

	// initialize the array before calculating the points to be plotted
	//
	initArray(0);

	// clear the drawing canvas
	//
	clearDrawingArea();
    }

    /**
     * draws sine wave to drawing canvas
     */
    public void drawSine() {

	// initialize the array before calculating the points to be plotted
	//
	initArray(1);

	// fill curve array with sinc wave values
	//
	int k = 0;
	double delta = 0.02;

	for (int i = xStart; i <= xEnd; k++, currLimit++, i++) {

	    xentry = ((i - datarect.x) * xratio) + xaxis.minimum;
	    yentry = Math.sin((2.0 * Math.PI * xentry)) / 
		(2.0 * Math.PI * xentry);
	    if (xentry > -delta && xentry < delta) {
		yentry = (double)1.0;
	    }
	    curveArray[2 * k] = (double)xentry;
	    curveArray[2 * k + 1] = (double)yentry;
	}

	// write curve data to data class so analysis can be performed
	//
	writeData();

	// draw waveform to canvas
	//
	updateScreen();
    }
    
    /**
     * draws cosine wave to drawing canvas
     */
    public void drawCosine() {
	
	// initialize the array before calculating the points to be plotted
	//
	initArray(2);
	
	// fill curve array with cosine wave values
	//
	int k = 0;

	for (int i = xStart; i <= xEnd; k++, currLimit++, i++) {

	    xentry = ((i - datarect.x) * xratio) + xaxis.minimum;
	    yentry = Math.cos((2.0 * Math.PI * xentry));
	    curveArray[2 * k] = (double)xentry;
	    curveArray[2 * k + 1] = (double)yentry;
	}

	// write curve data to data class so analysis can be performed
	//
	writeData();

	// draw waveform to canvas
	//
	updateScreen();
    }
    
    /**
     * draws sine wave to drawing canvas
     */
    public void drawSinc() {

	// initialize the array before calculating the points to be plotted
	//
	initArray(3);
	
	// fill curve array with sinc wave values
	//
	int k = 0;
	double delta = 0.02;

	for (int i = xStart; i <= xEnd; k++, currLimit++, i++) {

	    xentry = ((i - datarect.x) * xratio) + xaxis.minimum;
	    yentry = Math.sin((2.0 * Math.PI * xentry)) / 
		(2.0 * Math.PI * xentry);
	    if (xentry > -delta && xentry < delta) {
		yentry = (double)1.0;
	    }
	    curveArray[2 * k] = (double)xentry;
	    curveArray[2 * k + 1] = (double)yentry;
	}

	// write curve data to data class so analysis can be performed
	//
	writeData();

	// draw waveform to canvas
	//
	updateScreen();
    }
    
    /**
     * draws unit pulse function centered about the origin
     */
    public void drawUnit() {
	
	// initialize the array before calculating the points to be plotted
	//
	initArray(4);

	// fill curve array with unit pixel values
	//
	int k = 0;

	for (int i = xStart; i <= xEnd; k++, currLimit++, i++) {

	    xentry = ((i - datarect.x) * xratio) + xaxis.minimum;
	    if(xentry >= -0.5 && xentry <= 0.5) {
		yentry = 1.0;
	    } else {
		yentry = 0.0;
	    }
	    curveArray[2 * k] = (double)xentry;
	    curveArray[2 * k + 1] = (double)yentry;
	}

	// write curve data to data class so analysis can be performed
	//
	writeData();

	// draw waveform to canvas
	//
	updateScreen();
    }
    
    /**
     * draws triangle function centered about the origin
     */
    public void drawTriangle() {

	// initialize the array before calculating the points to be plotted
	//
	initArray(5);

	// fill curve array with unit pulse values
	//
	int k = 0;
	double slope1 = 1.0 / 0.5;
	double slope2 = -1.0 / 0.5;

	for (int i = xStart; i <= xEnd; k++, currLimit++, i++) {

	    xentry = ((i - datarect.x) * xratio) + xaxis.minimum;
	    if(xentry >= -0.5 && xentry <= 0.5) {
		if(xentry >= -0.5 && xentry <= 0.0) {
		    yentry = slope1 * (xentry + 0.5);
		} else {
		    yentry = slope2 * xentry + 1.0;
		}
	    } else {
		yentry = 0.0;
	    }
	    curveArray[2 * k] = (double)xentry;
	    curveArray[2 * k + 1] = (double)yentry;
	}

	// write curve data to data class so analysis can be performed
	//
	writeData();

	// draw waveform to canvas
	//
	updateScreen();
    }

    /**
     * draws an impulse centered about the origin
     */
    public void drawImpulse() {
	
	// initialize the array before calculating the points to be plotted
	//
	initArray(6);
	
	// fill curve array with unit pixel values
	//
	int k = 0;
	double delta = 0.02;

	for (int i = xStart; i <= xEnd; k++, currLimit++, i++) {

	    xentry = ((i - datarect.x) * xratio) + xaxis.minimum;
	    if(xentry > -delta && xentry < delta) {
		yentry = 1.0;
	    } else {
		yentry = 0.0;
	    }
	    curveArray[2 * k] = (double)xentry;
	    curveArray[2 * k + 1] = (double)yentry;
	}

	// write curve data to data class so analysis can be performed
	//
	writeData();

	// draw waveform to canvas
	//
	updateScreen();
    }
    
    /**
     * draws exponential function [exp(-t)] centered about the origin
     */
    public void drawExp() {
	
	// initialize the array before calculating the points to be plotted
	//
	initArray(7);
	
	// fill curve array with exponential wave values
	//
	int k = 0;

	for (int i = xStart; i <= xEnd; k++, currLimit++, i++) {

	    xentry = ((i - datarect.x) * xratio) + xaxis.minimum;
	    yentry = Math.exp(-xentry);
	    curveArray[2 * k] = (double)xentry;
	    curveArray[2 * k + 1] = (double)yentry;
	}

	// write curve data to data class so analysis can be performed
	//
	writeData();

	// draw waveform to canvas
	//
	updateScreen();
    }

    /**
     * draws exponential function [1-exp(-t)]centered about the origin
     */
   public void draw1Exp() {
	
	// initialize the array before calculating the points to be plotted
	//
	initArray(8);
	
	// fill curve array with 1-exponential wave values
	//
	int k = 0;

	for (int i = xStart; i <= xEnd; k++, currLimit++, i++) {

	    xentry = ((i - datarect.x) * xratio) + xaxis.minimum;
	    yentry = 1.0 - Math.exp(-xentry);
	    curveArray[2 * k] = (double)xentry;
	    curveArray[2 * k + 1] = (double)yentry;
	}

	// write curve data to data class so analysis can be performed
	//
	writeData();

	// draw waveform to canvas
	//
	updateScreen();
    }

    /**
     * draws the function [exp(-t) * cosine(t)] centered about the origin
     */
    public void drawExpCosine() {
	
	// initialize the array before calculating the points to be plotted
	//
	initArray(9);
	
	// fill curve array with exponential wave values
	//
	int k = 0;

	for (int i = xStart; i <= xEnd; k++, currLimit++, i++) {

	    xentry = ((i - datarect.x) * xratio) + xaxis.minimum;
	    yentry = Math.exp(-xentry) * Math.cos((2.0 * Math.PI * xentry));
	    curveArray[2 * k] = (double)xentry;
	    curveArray[2 * k + 1] = (double)yentry;
	}

	// write curve data to data class so analysis can be performed
	//
	writeData();

	// draw waveform to canvas
	//
	updateScreen();
    }

    /**
     * Empty implementation
     * 
     * @param cpData convolved data samples
     */
    public void drawCopy(Data cpData) {
	
    }
}

//
// end of file











